#
# Generalised Bitmap Module
#
# This makefile is for MS Visual C++ on Windows 32bit.
#
# Builds GBM.DLL which is compiled multithreaded
# Therefore all users should also be multithreaded

# directory of the platform makefiles
MAKEFILE_SRC = ..\makefile.w32

# Modify this line to point to the Independant JPEG groups library 6a/6b
# If you haven't got it, comment out the line instead.
IJG    = libjpeg

# Modify this line to point to the Open JPEG library 1.3
# If you haven't got it, comment out the line instead.
OPENJPEG = libojpeg

# Modify these 2 lines to point to the libpng library 1.2.31
# and the zlib 1.2.3.
# If you haven't got them, comment out the lines instead.
ZLIB   = zlib
LIBPNG = libpng

# Modify this line to point to the libtiff library 3.8.2
# If you haven't got it, comment out the line instead.
LIBTIFF = libtiff

# Documentation directory
DOCDIR = doc

# Installation directory: gbm.dll, gbm.lib and gbm.h files are copied to this directory
IDIR =  bin

# -------------------
# Configure lib usage
# -------------------

CJPEG    = /DENABLE_IJG
LIB_IJG  = $(IJG)\jpeg.lib

CJ2K     = /DENABLE_J2K
LIB_J2K  = $(OPENJPEG)\ojpeg.lib

# ZLIB is required for LIBPNG
CPNG     = /DENABLE_PNG
LIB_ZLIB = $(ZLIB)\z.lib
LIB_PNG  = $(LIBPNG)\png.lib

# ZLIB and IJG are required for LIBTIFF
CTIFF    = /DENABLE_TIF
LIB_TIFF = $(LIBTIFF)\tiff.lib


# ------------------
# Configure compiler
# ------------------

#
# Using MS Visual C++:
#
CC      = cl
CL      = link
CLIB    = lib

# Builds gbm objects which are compiled multithreaded
# Therefore all users should also be multithreaded

CWARNS         = /W3 /WX
CFLAGS         = /c $(CWARNS) /Gs /O2 /nologo /WX /MT /Zp4 /LD /D_CRT_SECURE_NO_DEPRECATE /DNDEBUG /DWIN32
CFLAGS_DBG     = /c $(CWARNS) /Gs /nologo /WX /MT /Zp4 /Zi /LD /D_CRT_SECURE_NO_DEPRECATE /DDEBUG /DWIN32
CFLAGS_EXE     = $(CFLAGS)
CFLAGS_EXE_DBG = $(CFLAGS_DBG)
CFLAGS_DLL     = /LD $(CFLAGS)     $(CJPEG) $(CJ2K) $(CPNG) $(CTIFF)
CFLAGS_DLL_DBG = /LD $(CFLAGS_DBG) $(CJPEG) $(CJ2K) $(CPNG) $(CTIFF)

LFLAGS         = /NOLOGO /INCREMENTAL:NO /STACK:0x6000
LFLAGS_DBG     = /NOLOGO /INCREMENTAL:NO /STACK:0x6000
LFLAGS_EXE     = $(LFLAGS)
LFLAGS_EXE_DBG = $(LFLAGS_DBG)
LFLAGS_DLL     = $(LFLAGS) /DLL
LFLAGS_DLL_DBG = $(LFLAGS_DBG) /DLL
LIBFLAGS       = /NOLOGO

# For debugging call nmake with debug=on
!ifdef debug
CFLAGS_EXE     = $(CFLAGS_EXE_DBG)
CFLAGS_DLL     = $(CFLAGS_DLL_DBG)
LFLAGS_EXE     = $(LFLAGS_EXE_DBG)
LFLAGS_DLL     = $(LFLAGS_DLL_DBG)
!endif

#

.SUFFIXES:	.c .obj

.c.obj:
		$(CC) $(CFLAGS_EXE) $*.c

#

all:    prep ijg j2k png tiff \
        gbmdllout \
        gbm.dll gbm.lib gbmmem.lib \
        gbmtoolsout \
        gbmhdr.exe \
        gbmmir.lib gbmref.exe \
        gbmrect.lib gbmsub.exe \
        gbmscale.lib gbmsize.exe \
        gbmerr.lib gbmtrunc.lib gbmht.lib gbmhist.lib gbmmcut.lib gbmbpp.exe \
        gbmcpal.exe  \
        gbmgamma.exe \
        gbmconv.exe \
        gbmver.exe

#

prep:
   @echo -----------------------------
   @echo JPEG support, via IJG library
   @echo -----------------------------
   @copy $(IJG)\jconfig.doc $(IJG)\jconfig.h
   @copy jmorecfg.h $(IJG)
   @echo ---------------------------------------
   @echo JPEG 2000 support, via OpenJPEG library
   @echo ---------------------------------------
   @echo --------------------------------
   @echo PNG support, via LIBPNG and ZLIB
   @echo --------------------------------
   @copy pngconf.h    $(LIBPNG)
   @copy pngusr.h     $(LIBPNG)
   @echo ---------------------------------------
   @echo TIFF support, via LIBTIFF, IJG and ZLIB
   @echo ---------------------------------------
   @echo.

ijg:
   @echo ---------------------
   @echo Compiling IJG library
   @echo ---------------------
!ifdef debug
   (cd $(IJG) && nmake /C /f $(MAKEFILE_SRC)\makeijg.ms debug=yes)
!else
   (cd $(IJG) && nmake /C /f $(MAKEFILE_SRC)\makeijg.ms)
!endif
   @echo.

#

j2k:
   @echo --------------------------
   @echo Compiling OpenJPEG library
   @echo --------------------------
!ifdef debug
   (cd $(OPENJPEG) && nmake /C /f $(MAKEFILE_SRC)\makej2k.ms debug=yes)
!else
   (cd $(OPENJPEG) && nmake /C /f $(MAKEFILE_SRC)\makej2k.ms)
!endif
   @echo.

#

png:
   @echo ----------------------
   @echo Compiling ZLIB library
   @echo ----------------------
!ifdef debug
   (cd $(ZLIB)   && nmake /C /f $(MAKEFILE_SRC)\makezlib.ms debug=yes)
!else
   (cd $(ZLIB)   && nmake /C /f $(MAKEFILE_SRC)\makezlib.ms)
!endif
   @echo.
   @echo ------------------------
   @echo Compiling LIBPNG library
   @echo ------------------------
!ifdef debug
   (cd $(LIBPNG) && nmake /C /f $(MAKEFILE_SRC)\makepng.ms debug=yes)
!else
   (cd $(LIBPNG) && nmake /C /f $(MAKEFILE_SRC)\makepng.ms)
!endif
   @echo.

#

tiff:
   @echo -------------------------
   @echo Compiling LIBTIFF library
   @echo -------------------------
!ifdef debug
   (cd $(LIBTIFF) && nmake /C /f $(MAKEFILE_SRC)\maketif.ms debug=yes)
!else
   (cd $(LIBTIFF) && nmake /C /f $(MAKEFILE_SRC)\maketif.ms)
!endif
   @echo.

#

gbmdllout:
   @echo ---------------------
   @echo Compiling GBM library
   @echo ---------------------

gbmtoolsout:
   @echo -------------------
   @echo Compiling GBM tools
   @echo -------------------

#

gbm.dll:  gbm.obj    gbmpbm.obj  gbmpgm.obj gbmppm.obj gbmpnm.obj  \
          gbmbmp.obj gbmtga.obj  gbmkps.obj gbmiax.obj gbmpcx.obj  \
          gbmtif.obj gbmlbm.obj  gbmvid.obj gbmgif.obj gbmxbm.obj  \
          gbmspr.obj gbmpsg.obj  gbmgem.obj gbmcvp.obj gbmjpg.obj  \
          gbmj2k.obj gbmpng.obj gbmxpm.obj  gbmxpmcn.obj gbmhelp.obj gbmmap.obj gbmmem.obj
          $(CL) $(LFLAGS_DLL) /OUT:$@ @<<
			$** $(LIB_IJG) $(LIB_J2K) $(LIB_ZLIB) $(LIB_PNG) $(LIB_TIFF)
<<

gbm.lib:
          $(CLIB) $(LIBFLAGS) /OUT:$@ gbm.dll

gbm.obj:  gbm.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h gbmpbm.h gbmpgm.h gbmpnm.h gbmppm.h gbmbmp.h gbmtga.h gbmkps.h gbmiax.h gbmpcx.h gbmtif.h gbmlbm.h gbmvid.h gbmgif.h gbmxbm.h gbmspr.h gbmpsg.h gbmgem.h gbmcvp.h gbmjpg.h gbmpng.h gbmxpm.h gbmxpmcn.h gbmj2k.h
          $(CC) $(CFLAGS_DLL) $*.c

# ------------
# File formats
# ------------

gbmpbm.obj:	gbmpbm.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmpgm.obj:	gbmpgm.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmpnm.obj:	gbmpbm.c gbm.h gbmhelp.h gbmdesc.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmppm.obj:	gbmppm.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmbmp.obj:	gbmbmp.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmtga.obj:	gbmtga.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmkps.obj:	gbmkps.c gbm.h gbmhelp.h gbmdesc.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmiax.obj:	gbmiax.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmpcx.obj:	gbmpcx.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmlbm.obj:	gbmlbm.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmvid.obj:	gbmvid.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmgif.obj:	gbmgif.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmxbm.obj:	gbmxbm.c gbm.h gbmhelp.h gbmdesc.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmxpm.obj:	gbmxpm.c gbm.h gbmhelp.h gbmdesc.h gbmmap.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmxpmcn.obj:	gbmxpmcn.c gbmxpmcn.h gbm.h gbmmap.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmspr.obj:	gbmspr.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmpsg.obj:	gbmpsg.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmgem.obj:	gbmgem.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmcvp.obj:	gbmcvp.c gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

# ------------------

gbmhelp.obj:	gbmhelp.c gbm.h gbmmem.h
		$(CC) $(CFLAGS_DLL) $*.c

gbmmap.obj:	gbmmap.c gbm.h gbmmap.h
		$(CC) $(CFLAGS_DLL) $*.c

# ------------------

gbmjpg.obj:	gbmjpg.c gbmjpg.h gbm.h gbmhelp.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) /I$(IJG) $*.c

gbmj2k.obj:	gbmj2k.c gbmj2k.h gbm.h gbmhelp.h gbmdesc.h
		$(CC) $(CFLAGS_DLL) /DOPJ_STATIC /DUSE_JPWL /I$(OPENJPEG) $*.c

gbmpng.obj:	gbmpng.c gbm.h gbmhelp.h gbmmap.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) /I$(LIBPNG) /I$(ZLIB) $*.c

gbmtif.obj:	gbmtif.c gbm.h gbmhelp.h gbmmap.h gbmdesc.h gbmmem.h
		$(CC) $(CFLAGS_DLL) /I$(IJG) /I$(ZLIB) /I$(LIBTIFF) $*.c


# ------------
# Bitmap tools
# ------------

gbmtool.obj:    gbmtool.c gbmtool.h gbm.h

gbmtw32.obj:    gbmtw32.c gbmtool.h gbm.h

gbmtool.lib:	gbmtool.obj gbmtw32.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

#

gbmmem.obj:	gbmmem.c gbm.h gbmmem.h

gbmmem.lib:	gbmmem.obj
            $(CLIB) $(LIBFLAGS) /OUT:$@ $**

#

gbmhdr.exe:	gbmhdr.obj gbm.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmhdr.obj:	gbmhdr.c gbm.h

#

gbmmir.lib:	gbmmir.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmmir.obj:	gbmmir.c gbmmir.h gbmmem.h

gbmref.exe:	gbmref.obj gbm.lib gbmmem.lib gbmmir.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmref.obj:	gbmref.c gbm.h gbmmem.h gbmmir.h

#

gbmrect.lib:	gbmrect.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmrect.obj:	gbmrect.c

gbmsub.exe:	gbmsub.obj gbm.lib gbmmem.lib gbmrect.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmsub.obj:	gbmsub.c gbm.h gbmmem.h gbmrect.h

#

gbmerr.lib:	gbmerr.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmerr.obj:	gbmerr.c gbmmem.h

#

gbmscale.lib:	gbmscale.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmscale.obj:	gbmscale.c gbmmem.h

gbmsize.exe:	gbmsize.obj gbm.lib gbmmem.lib gbmscale.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmsize.obj:	gbmsize.c gbm.h gbmmem.h gbmscale.h

#

gbmtrunc.lib:	gbmtrunc.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmtrunc.obj:	gbmtrunc.c gbmtrunc.h

#

gbmht.lib:	gbmht.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmht.obj:	gbmht.c gbmht.h

#

gbmhist.lib:	gbmhist.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmhist.obj:	gbmhist.c gbmmem.h

#

gbmmcut.lib:	gbmmcut.obj
                $(CLIB) $(LIBFLAGS) /OUT:$@ $**

gbmmcut.obj:	gbmmcut.c gbmmem.h

#

gbmbpp.exe:	gbmbpp.obj gbm.lib gbmmem.lib gbmerr.lib gbmtrunc.lib gbmht.lib gbmhist.lib gbmmcut.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmbpp.obj:	gbmbpp.c gbm.h gbmmem.h gbmerr.h gbmtrunc.h gbmht.h gbmhist.h gbmmcut.h

#

gbmcpal.exe:	gbmcpal.obj gbm.lib gbmmem.lib gbmhist.lib gbmmcut.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmcpal.obj:	gbmcpal.c gbm.h gbmmem.h gbmhist.h gbmmcut.h

#

gbmgamma.exe:	gbmgamma.obj gbm.lib gbmmem.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmgamma.obj:	gbmgamma.c gbm.h gbmmem.h

#

gbmconv.exe:	gbmconv.obj gbm.lib gbmmem.lib gbmtool.lib
		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmconv.obj:	gbmconv.c gbm.h gbmmem.h

#
# TODO: On Windows we load gbm.dll dynamically, so we don't need gbm.lib.
# For the moment we link against gbm.lib.

gbmver.exe:	gbmverw32.obj
  		$(CL) $(LFLAGS_EXE) /OUT:$@ $**

gbmverw32.obj:	gbmverw32.c gbm.h


# ------------------------
# Build management targets
# ------------------------

clean:  prep
	-del /F $(IJG)\jmorecfg.h $(LIBPNG)\pngconf.h $(LIBPNG)\pngusr.h 2>nul
	-del /F /Q *.obj 2>nul
	-del /F /Q *.exp 2>nul
	-del /F /Q *.pdb 2>nul
	(cd $(IJG)      && nmake /C /f $(MAKEFILE_SRC)\makeijg.ms  clean)
	(cd $(OPENJPEG) && nmake /C /f $(MAKEFILE_SRC)\makej2k.ms  clean)
	(cd $(ZLIB)     && nmake /C /f $(MAKEFILE_SRC)\makezlib.ms clean)
	(cd $(LIBPNG)   && nmake /C /f $(MAKEFILE_SRC)\makepng.ms  clean)
	(cd $(LIBTIFF)  && nmake /C /f $(MAKEFILE_SRC)\maketif.ms  clean)

clobber: prep
	 -del /F $(IJG)\jmorecfg.h $(LIBPNG)\pngconf.h $(LIBPNG)\pngusr.h 2>nul
	 -del /F /Q *.obj 2>nul
	 -del /F /Q *.exp 2>nul
 	 -del /F /Q *.pdb 2>nul
	 -del /F /Q *.exe 2>nul
	 -del /F /Q *.lib 2>nul
	 -del /F /Q *.dll 2>nul
	 -del /F /Q $(IDIR)\* 2>nul
	 (cd $(IJG)      && nmake /C /f $(MAKEFILE_SRC)\makeijg.ms  clobber)
	 (cd $(OPENJPEG) && nmake /C /f $(MAKEFILE_SRC)\makej2k.ms  clobber)
	 (cd $(ZLIB)     && nmake /C /f $(MAKEFILE_SRC)\makezlib.ms clobber)
	 (cd $(LIBPNG)   && nmake /C /f $(MAKEFILE_SRC)\makepng.ms  clobber)
	 (cd $(LIBTIFF)  && nmake /C /f $(MAKEFILE_SRC)\maketif.ms  clobber)

#

install:
	     (cd $(IJG)      && nmake /C /f $(MAKEFILE_SRC)\makeijg.ms  install)
	     (cd $(OPENJPEG) && nmake /C /f $(MAKEFILE_SRC)\makej2k.ms  install)
	     (cd $(ZLIB)     && nmake /C /f $(MAKEFILE_SRC)\makezlib.ms install)
	     (cd $(LIBPNG)   && nmake /C /f $(MAKEFILE_SRC)\makepng.ms  install)
	     (cd $(LIBTIFF)  && nmake /C /f $(MAKEFILE_SRC)\maketif.ms  install)
         copy gbm.h      $(IDIR)
         copy gbmerr.h   $(IDIR)
         copy gbmhist.h  $(IDIR)
         copy gbmht.h    $(IDIR)
         copy gbmmcut.h  $(IDIR)
         copy gbmmir.h   $(IDIR)
         copy gbmrect.h  $(IDIR)
         copy gbmscale.h $(IDIR)
         copy gbmtrunc.h $(IDIR)
         copy gbmtool.h  $(IDIR)
         copy gbmmem.h   $(IDIR)
         copy *.lib      $(IDIR)
         copy *.exe      $(IDIR)
         copy *.dll      $(IDIR)
         copy $(DOCDIR)\readme_gbm.txt $(IDIR)

#

exepack:

#

package:
	    (cd $(IJG)      && nmake /C /f $(MAKEFILE_SRC)\makeijg.ms  package)
	    (cd $(OPENJPEG) && nmake /C /f $(MAKEFILE_SRC)\makej2k.ms  package)
	    (cd $(ZLIB)     && nmake /C /f $(MAKEFILE_SRC)\makezlib.ms package)
	    (cd $(LIBPNG)   && nmake /C /f $(MAKEFILE_SRC)\makepng.ms  package)
	    (cd $(LIBTIFF)  && nmake /C /f $(MAKEFILE_SRC)\maketif.ms  package)
        -del /F $(IDIR)\gbm.zip 2>nul
        zip -9 $(IDIR)\gbm.zip *.exe *.dll *.lib gbm.h gbmerr.h gbmhist.h  \
                               gbmht.h gbmmcut.h gbmmir.h gbmrect.h  \
                               gbmscale.h gbmtrunc.h gbmtool.h gbmmem.h \
                               $(DOCDIR)\readme_gbm.txt

